/********************************************************************************
 Author : CAC (CustomerApplications Center, Asia) 

 Date : 2012-01-04

 File name : AD7606.c

 Description : test the serial port operation of AD7606

 Hardware plateform : ADuC7026_DEMO_V1.2 + EVAL-AD7606EDZ_Rev.A1  	
********************************************************************************/

//#include "ADuC7026.h"
//#include "ADuC7026Driver.h"
#include "main.h"
#include "AD7606.h"


/*---- Transplanted ADu7026 OUTPUT configuration to MAX32670's ----*/
void ADuC7026OutputBit(uint32_t pin_mask, mxc_gpio_vssel_t IO_level)
{
    int retval = E_NO_ERROR;
    uint32_t vsel;
    //IO configuration
    mxc_gpio_cfg_t gpio_cfg_struct;

    gpio_cfg_struct.port 	= MXC_GPIO0;
    gpio_cfg_struct.mask 	= pin_mask;//pin_mask;
    gpio_cfg_struct.func	= MXC_GPIO_FUNC_OUT;
	gpio_cfg_struct.pad		= MXC_GPIO_PAD_NONE;
	//	gpio_cfg_struct.vssel	= IO_level;		//malfunction!!!
	vsel = (IO_level ? 0xffffffff : 0x00000000);

    if (MXC_GPIO_Config(&gpio_cfg_struct) != E_NO_ERROR) {
	   retval = E_UNKNOWN;
	   printf("ERROR!!\r\n");
	}
    MXC_GPIO_OutPut(gpio_cfg_struct.port,gpio_cfg_struct.mask,vsel);
}

/*---- Transplanted ADu7026 INPUT configuration to MAX32670's ----*/
uint8_t ADuC7026InputBit(uint32_t pin_mask)
{
	uint8_t data;
    //IO configuration
    mxc_gpio_cfg_t gpio_cfg_struct;

    gpio_cfg_struct.port 	= MXC_GPIO0;
    gpio_cfg_struct.mask 	= pin_mask;
    gpio_cfg_struct.func	= MXC_GPIO_FUNC_IN;
	gpio_cfg_struct.pad		= MXC_GPIO_PAD_PULL_UP;
//	gpio_cfg_struct.vssel	= IO_level;

    MXC_GPIO_Config(&gpio_cfg_struct);

    data = MXC_GPIO_InGet(gpio_cfg_struct.port,gpio_cfg_struct.mask) ? 0x01 : 0x00;

    return data;
}


void AD7606Initialization(void)
{
	MXC_Delay(MSEC(1));
	ADuC7026OutputBit(AD7606_RESET, 0);
	ADuC7026OutputBit(AD7606_CS, 1);
	ADuC7026OutputBit(AD7606_CNVSTA | AD7606_CNVSTB, 1);
	ADuC7026OutputBit(AD7606_SCLK, 1);
	ADuC7026InputBit(AD7606_DOUTA);
	ADuC7026InputBit(AD7606_DOUTB);
	ADuC7026InputBit(AD7606_BUSY);
	MXC_Delay(MSEC(1));


	/*---- Source code ----*/
//	ADuC7026DelayMs(1);
//	ADuC7026OutputBit(AD7606_RESET, 0);
//	ADuC7026OutputBit(AD7606_CS, 1);
//	ADuC7026OutputBit(AD7606_CNVST, 1);
//	ADuC7026OutputBit(AD7606_SCLK, 1);
//	ADuC7026InputBit(AD7606_DOUTA);
//	ADuC7026InputBit(AD7606_DOUTB);
//	ADuC7026InputBit(AD7606_BUSY);
//	ADuC7026DelayMs(1);
}

void AD7606Reset(void)
{
	ADuC7026OutputBit(AD7606_RESET, 1);
	MXC_Delay(USEC(1));
	ADuC7026OutputBit(AD7606_RESET, 0);
	MXC_Delay(USEC(1));

	/*---- Source code ----*/
//	ADuC7026OutputBit(AD7606_RESET, 1);
//	ADuC7026DelayUs(1);
//	ADuC7026OutputBit(AD7606_RESET, 0);
//	ADuC7026DelayUs(1);
}

void AD7606ReadOneSample(unsigned short int *DoutA, unsigned short int *DoutB, unsigned char Channels)
{
	unsigned char j, k;
	unsigned short int TempA, TempB;

	unsigned char Busy;

	ADuC7026OutputBit(AD7606_CNVSTA | AD7606_CNVSTB, 0);
	MXC_Delay(USEC(1));
	ADuC7026OutputBit(AD7606_CNVSTA | AD7606_CNVSTB, 1);
	MXC_Delay(USEC(1));

	Busy=ADuC7026InputBit(AD7606_BUSY);

	while(Busy==1)
	{
		MXC_Delay(USEC(1));
		Busy=ADuC7026InputBit(AD7606_BUSY);
	}

	ADuC7026OutputBit(AD7606_CS, 0);

	for(j=0; j<Channels; j++)
	{
		TempA=0;
		TempB=0;

		for(k=0; k<16; k++)
		{
			ADuC7026OutputBit(AD7606_SCLK, 0);

			TempA=(TempA<<1) + ADuC7026InputBit(AD7606_DOUTA);
			TempB=(TempB<<1) + ADuC7026InputBit(AD7606_DOUTB);

			ADuC7026OutputBit(AD7606_SCLK, 1);
		}
		
		DoutA[j]=TempA;
		DoutB[j]=TempB;
	}

	ADuC7026OutputBit(AD7606_CS, 1);

	/*---- Source code ----*/
//	ADuC7026OutputBit(AD7606_CNVST, 0);
//	ADuC7026DelayUs(1);
//	ADuC7026OutputBit(AD7606_CNVST, 1);
//	ADuC7026DelayUs(1);
//
//	Busy=ADuC7026InputBit(AD7606_BUSY);
//
//	while(Busy==1)
//	{
//		ADuC7026DelayUs(1);
//		Busy=ADuC7026InputBit(AD7606_BUSY);
//	}
//
//	ADuC7026OutputBit(AD7606_CS, 0);
//
//	for(j=0; j<Channels; j++)
//	{
//		TempA=0;
//		TempB=0;
//
//		for(k=0; k<16; k++)
//		{
//			ADuC7026OutputBit(AD7606_SCLK, 0);
//
//			TempA=(TempA<<1) + ADuC7026InputBit(AD7606_DOUTA);
//			TempB=(TempB<<1) + ADuC7026InputBit(AD7606_DOUTB);
//
//			ADuC7026OutputBit(AD7606_SCLK, 1);
//		}
//
//		DoutA[j]=TempA;
//		DoutB[j]=TempB;
//	}
//
//	ADuC7026OutputBit(AD7606_CS, 1);
}
