/*******************************************************************************
 * Copyright (C) 2016 Maxim Integrated Products, Inc., All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated
 * Products, Inc. shall not be used except as stated in the Maxim Integrated
 * Products, Inc. Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products, Inc. retains all
 * ownership rights.
 *
 * $Date: 2018-09-05 16:46:11 -0500 (Wed, 05 Sep 2018) $
 * $Revision: 37695 $
 *
 ******************************************************************************/

/**
 * @file    main.c
 * @brief   Hello World!
 * @details This example uses the UART to print to a terminal and flashes an LED.
 */

/***** Includes *****/
#include "main.h"
#include <math.h>
#include "AD7606.h"
#include "tmr.h"

/***** Definitions *****/
// Parameters for PWM output
#define CLOCK_SOURCE    MXC_TMR_8M_CLK     // must be mxc_tmr_clock_t

// Parameters for Continuous timer
#define CONT_FREQ       2                   // (Hz)
#define CONT_TIMER      MXC_TMR3            // Can be MXC_TMR0 through MXC_TMR5

/***** Globals *****/
uint16_t	DataA[8] = {0};
uint16_t	DataB[8] = {0};
uint16_t	SampleCount = 0;
uint8_t		ChannelCount = 0;

uint16_t	i, j;

volatile Water_Quliaty_StructDef wq_info;

/***** Functions *****/
void ContinuousTimer(void);

// Toggles GPIO when continuous timer repeats
void ContinuousTimerHandler(void)
{
    // Clear interrupt
    MXC_TMR_ClearFlags(CONT_TIMER);
    MXC_GPIO_OutToggle(led_pin[1].port, led_pin[1].mask);
    wq_info.update_flag = 1;
}

void ContinuousTimer(void)
{
    // Declare variables
    mxc_tmr_cfg_t tmr;
    uint32_t periodTicks = MXC_TMR_GetPeriod(CONT_TIMER, CLOCK_SOURCE, 128, CONT_FREQ);

    /*
    Steps for configuring a timer for PWM mode:
    1. Disable the timer
    2. Set the prescale value
    3  Configure the timer for continuous mode
    4. Set polarity, timer parameters
    5. Enable Timer
    */

    MXC_TMR_Shutdown(CONT_TIMER);

    tmr.pres = TMR_PRES_128;
    tmr.mode = TMR_MODE_CONTINUOUS;
    tmr.clock = CLOCK_SOURCE;
    tmr.cmp_cnt = periodTicks;      //SystemCoreClock*(1/interval_time);
    tmr.pol = 0;

    MXC_TMR_Init(CONT_TIMER, &tmr, true);
    MXC_TMR_EnableInt(CONT_TIMER);
    MXC_TMR_Start(CONT_TIMER);

    printf("Continuous timer started.\n\n");
}


// *****************************************************************************
int main(void)
{
    int retval;
    int error;
    int count = 0;
    int16_t	value = 0;
    float vol_chx[8] = {0};

    __NOP();
    /***********   Timer3   **********/
    NVIC_SetVector(TMR3_IRQn, ContinuousTimerHandler);
    NVIC_EnableIRQ(TMR3_IRQn);
    ContinuousTimer();

    /***********   UART0   **********/
    //IO configuration and remapping
    mxc_gpio_cfg_t gpio_cfg_uart0x_disable	= { MXC_GPIO0, (MXC_GPIO_PIN_8  | MXC_GPIO_PIN_9 ),  MXC_GPIO_FUNC_IN, MXC_GPIO_PAD_PULL_UP };
    mxc_gpio_cfg_t gpio_cfg_uart0x			= { MXC_GPIO0, (MXC_GPIO_PIN_24 | MXC_GPIO_PIN_25),  MXC_GPIO_FUNC_ALT2, MXC_GPIO_PAD_PULL_UP };


    //Configure UART baud rate and clock source
    MXC_UART_Init ((mxc_uart_regs_t*)MXC_UART0, 115200, (mxc_uart_clock_t)MXC_UART_APB_CLK);

    //Reset the GPIO and Disable the GPIO8 and 9 as a not UART0's port.
    MXC_GPIO_Config(&gpio_cfg_uart0x_disable);		//Close the UART0 default IO
    MXC_GPIO_Config(&gpio_cfg_uart0x);

    /***********   AD7606   **********/
    AD7606Initialization();

	AD7606Reset();

	SampleCount=100;
	ChannelCount=8;

    /***********   while   **********/
    while (1) {

        if(wq_info.update_flag == 1)
        {
        	LED_On(0);
            wq_info.update_flag = 0;
            printf(" ____  ____  ____    ____  ____   __  ____  ____ \n");			//Generate an Logo
            printf("(_  _)(    \\/ ___)  (  _ \\(  _ \\ /  \\(  _ \\(  __) \n");			//Generate an Logo
            printf("  )(   ) D (\\___ \\   ) __/ )   /(  O )) _ ( ) _) \n");			//Generate an Logo
            printf(" (__) (____/(____/  (__)  (__\\_) \\__/(____/(____) \n");			//Generate an Logo
            printf("*************************************************\n");			//Generate an Logo

    		AD7606ReadOneSample(DataA, DataB, ChannelCount);
    		MXC_Delay(USEC(10));

    		for(j=0; j<ChannelCount; j++)
    		{
    //			printf("DoutA[%d] = %04x[%d] , DoutB[%d] = %04x[%d] .\r\n", j, DataA[j],DataA[j], j, DataB[j], DataB[j]);
    			value = (DataA[j] & 0x8000) ? (~(DataA[j] - 1)) : DataA[j];
    			vol_chx[j] = value * 0.000001 * 152 * ((DataA[j] & 0x8000) ? (-1) : (1));
    //			printf("ch_%d = %d,%f\r\n",j,value,vol_chx[j]);
    		}
    //		printf("\r\n");

    		/* Water temperature sensor calculation*/
			#define R_TEMP25    50.0		//50kohm under 25 degree C
			#define TEMP25      25.0
			#define Kelven_25C  273.15
			#define R_BIAS      100.0        //10kohm
			#define B_NTC_50K   3580

    		wq_info.w_f_temp = 1.0 / (1.0 / (Kelven_25C + TEMP25) + 1.0 / B_NTC_50K * log((vol_chx[0]/((vol_chx[0] - vol_chx[1]) / R_BIAS) - R_BIAS)/R_TEMP25)) - Kelven_25C;
    		printf("water TMP = %2.2f\r\n",wq_info.w_f_temp);

    		/* Water TDS sensor calculation*/
			#define	K_VALUE		1.0 * 2

    		wq_info.w_f_vol = (1.0 + 0.02 * (wq_info.w_f_temp - 25.0)) * vol_chx[2];
    		wq_info.w_f_tds = K_VALUE * (66.71 * pow(wq_info.w_f_vol,3) - 127.93 * pow(wq_info.w_f_vol,2) + 428.7 * wq_info.w_f_vol);

    		printf("water TDS = %2.2fppm[%2.2f]\r\n",((wq_info.w_f_tds > 0.0) ? wq_info.w_f_tds : 0.0),wq_info.w_f_tds);

			LED_Off(0);
        }
//      MXC_Delay(400000);
//    	MXC_Delay(500000);
//      printf("count = %d\n", count++);
    }
}



/* ---- Code dustbin ----*/

//	for(i=0; i<SampleCount; i++)
//	{
//		printf("Sample%d\r\n", i);
//
//		AD7606ReadOneSample(DataA, DataB, ChannelCount);
//		MXC_Delay(USEC(10));
//
//		for(j=0; j<ChannelCount; j++)
//		{
//			printf("DoutA[%d] = %04x , DoutB[%d] = %04x .\r\n", j, DataA[j], j, DataB[j]);
//		}
//		printf("\r\n");
//	}


//ADuC7026OutputBit(MXC_GPIO_PIN_29, 0);
//printf("It seems work! 0x%08x\r\n",ADuC7026InputBit(MXC_GPIO_PIN_28));
////    	MXC_GPIO0->out = 0xffffffff;
//printf("%x\r\n",MXC_GPIO_InGet(MXC_GPIO0,MXC_GPIO_PIN_8 | MXC_GPIO_PIN_28));
//MXC_GPIO_OutPut(MXC_GPIO0,MXC_GPIO_PIN_23,0x00000000);

//ADuC7026OutputBit(MXC_GPIO_PIN_29, 1);
//printf("It seems work! 0x%08x\r\n",ADuC7026InputBit(MXC_GPIO_PIN_28));
////        MXC_GPIO0->out = 0x00000000;
//printf("%x\r\n",MXC_GPIO_InGet(MXC_GPIO0,MXC_GPIO_PIN_8 | MXC_GPIO_PIN_28));
//MXC_GPIO_OutPut(MXC_GPIO0,MXC_GPIO_PIN_23,0xffffffff);
